//  Matrix 16 x 16
//  Mit dem EPS8266 Chip auf der Platine
//  
//  Unter Verwendung der IoT-Library ab Version 1.03


#include <all_IoT.h>
#include <all_Matrix.h>


#define WIFI_TIMEOUT       30000                                // WLAN Verbindung jede ... ms. prüfen
#define BRIGHTNESS_STEP    15                                   // Helligkeit um diesen Wert erhöhen/verringern
#define SPEED_STEP         10                                   // Geschwindigkeit um diesen Wert erhöhen/verringern


unsigned long LastWLANtime = 0;
String        modes        = "";
byte          lastHour     = 0;
byte          lastMinute   = 0;


void setup ()
{
  IoT_Init(false);                                                  // EPS8266 initialisieren (ohne OLED/ADC)
  t_TerminalInit();                                                 // Terminal initialisieren
  matrix_Init();                                                    // Matrix initialisieren
  matrix_Pixel.setBrightness(16);                                   // Helligkeit (0..255)
  matrix_Clear();
  matrix_Show();
  t_TerminalClearScreen();
  Serial.println("Matrix 16 x 16");
  Serial.println("--------------");
  Serial.println("");
  matrix_DrawString("PARA", 1,  1, matrix_Color.FontRed, matrix_Color.FontGreen, matrix_Color.FontBlue, matrix_Color.FontWhite, true);
  matrix_DrawString("CLR?", 1,  9, matrix_Color.FontRed, matrix_Color.FontGreen, matrix_Color.FontBlue, matrix_Color.FontWhite, true);
  matrix_Show();
  IoT_WaitKeypress(3000, false);                                    // 3 s warten, in der Zeit kann mit Taster die Konfig zurückgesetzt werden
  matrix_Clear();
  matrix_Show();
  if (IoT_Keypress())                                               // Wenn Taster gedrückt wird, Konfiguration im EEPROM löschen
  {
    IoT_EEPROMclear();                                              // Alle Parameter aus dem EEPROM löschen
    IoT_EEPROMupdate();                                             // EEPROM aktualisieren
    Serial.print("Matrix Parameter aus EEPROM gelöscht. ");
    Serial.println("Matrix wird neu gestartet...");
    delay(100);                                                     // 100 ms. warten
    IoT_WaitNoKeypress();                                           // Warten bis der Taster losgelassen wurde
    t_Restart();                                                    // Neustart durchführen
  }
  if (matrix_ParameterLoad())                                       // Parameter-Block aus EEPROM laden
  {
    Serial.println("Matrix Parameter aus EEPROM geladen.");
    matrix_ParameterApply();                                        // Parameter-Block in die aktuelle Konfiguration übernehmen
  }
  else
  {
    Serial.println("Matrix Parameter im EEPROM nicht vorhanden.");
  }
  Serial.println("WLAN wird gestartet...");
  matrix_WlanAutoConnect();
  matrix_Clear();
  matrix_DrawString("LOOK", 1,  1, matrix_Color.FontRed, matrix_Color.FontGreen, matrix_Color.FontBlue, matrix_Color.FontWhite, true);
  matrix_DrawString("NTP", 1,  9, matrix_Color.FontRed, matrix_Color.FontGreen, matrix_Color.FontBlue, matrix_Color.FontWhite, true);
  matrix_Show();  
  IoT_NTPinit();                                                    // NTP-Timeserver initialisieren
  DHT11_Init();
  Serial.println("NTP Time-Server wurde gefunden.");

  Serial.println("Konfiguration wird vorbereitet...");
  Serial.print("HTTP Server wird eingerichtet...");
  IoT_WebServer.on("/", handleRoot);                                // Browser-Handler (siehe unten) für das Stammverzeichnis
  IoT_WebServer.on("/Reload", handleRoot);                          // Handler für Reload
  IoT_WebServer.on("/NameChange",     handleNameChange);            // Handler für Namen ändern
  IoT_WebServer.on("/NetworkChange",  handleNetworkChange);         // Handler für Netzwerkeinstellungen ändern
  IoT_WebServer.on("/DynamicIP",      handleDynamicIP);             // Handler für dynamische IP-Adresse
  IoT_WebServer.on("/SavePrefs",      handleSavePrefs);             // Handler für Einstellungen dauerhaft speichern
  IoT_WebServer.on("/StandardPrefs",  handleStandardPrefs);         // Handler für Standardeinstellungen widerherstellen
  IoT_WebServer.on("/MatrixKonfig",   handleMatrixKonfig);          // Handler für Matrix konfigurieren
  IoT_WebServer.on("/MatrixChange",   handleMatrixChange);          // Handler für Matrix Konfiguration ändern
  IoT_WebServer.on("/ALL3500",        handleALL3500);               // Handler für ALL-3500 Sensoren konfigurieren
  IoT_WebServer.on("/ALL3500Change",  handleALL3500Change);         // Handler für ALL-3500 URL ändern
  IoT_WebServer.on("/ModulesOn",      handleModulesOn);             // Handler für Verfügbare Module einblenden
  IoT_WebServer.on("/ModulesOff",     handleModulesOff);            // Handler für Verfügbare Module ausblenden
  IoT_WebServer.on("/ColorKonfig",    handleColorKonfig);           // Handler für Farbeinstellungen konfigurieren
  IoT_WebServer.on("/ColorChange",    handleColorChange);           // Handler für Farbeinstellungen ändern
  IoT_WebServer.begin();                                            // ab jetzt "horcht" der Server auf HTTP-Anfragen
  IoT_WebServer.handleClient();                                     // Bediene die http Anfragen
  Serial.println("HTTP Server wurde gestartet.");

  matrix_Clear();
  matrix_Show();
  matrix_Pixel.setBrightness(matrix_Color.Brightness);              // Helligkeit (0..255)
 }

void loop ()
{
  IoT_Idle();
  IoT_WebServer.handleClient();                                     // Bediene die http Anfragen

  int h = hour();
  int m = minute();
  int s = second();
  
  if (h != lastHour)                                              // Aktionen ein mal pro Stunde durchführen
  {
    all3500_ReadSensorData(matrix_ALL3500URL, true);
    lastHour = h;
  }
  if (m != lastMinute)                                            // Aktionen ein mal pro Miniute durchführen
  {
    lastMinute = m;
  }
  
  if (IoT_Keypress())                                             // Wenn Taster gedrückt wird, Konfiguration zeigen
  {
    matrix_Clear();
    String ip = "IP: " + IoT_WLANaddress(true) + "  GW: " + IoT_WLANgateway(true) + "  NT: " + IoT_WLANsubnet(true);
    Serial.println(IoT_NTPdatetime() + " (" + IoT_NTPdaylightSavingTime(true) + ")");
    Serial.print("WiFi is ");
    Serial.print(IoT_WLANconnected() ? "connected" : "not connected");
    Serial.println(". Uptime: " + IoT_WLANuptime(true) + " seit " + IoT_WLANstartDatetime());
    Serial.println(ip);                                           // IP-Information
    String Ipaddr = replace(IoT_WLANaddress(true), ".", "");
    matrix_DrawString(mid(Ipaddr, 1, 4), 0,  0, matrix_Color.FontRed, matrix_Color.FontGreen, matrix_Color.FontBlue, matrix_Color.FontWhite, true);
    matrix_DrawString(mid(Ipaddr, 5, 4), 0,  5, matrix_Color.FontRed, matrix_Color.FontGreen, matrix_Color.FontBlue, matrix_Color.FontWhite, true);
    matrix_DrawString(mid(Ipaddr, 9, 4), 0, 10, matrix_Color.FontRed, matrix_Color.FontGreen, matrix_Color.FontBlue, matrix_Color.FontWhite, true);
    matrix_Plot(11,  4, matrix_Color.SeparatorRed, matrix_Color.SeparatorGreen, matrix_Color.SeparatorBlue, matrix_Color.SeparatorWhite);
    matrix_Plot( 7,  9, matrix_Color.SeparatorRed, matrix_Color.SeparatorGreen, matrix_Color.SeparatorBlue, matrix_Color.SeparatorWhite);
    matrix_Plot( 3, 14, matrix_Color.SeparatorRed, matrix_Color.SeparatorGreen, matrix_Color.SeparatorBlue, matrix_Color.SeparatorWhite);
    matrix_Show();                                                // Anzeige aktualisieren
    while (IoT_Keypress())
    {
      IoT_Idle();
      delay(10);
    }
  }
  else if (matrix_Keypress())                                     // Wenn Matrix-Taster gedrückt wird, nächste virtuelle Seite aufrufen
  {
    matrix_Clear();
    matrix_DrawString("NEXT", 1,  1, matrix_Color.FontRed, matrix_Color.FontGreen, matrix_Color.FontBlue, matrix_Color.FontWhite, true);
    matrix_DrawString("SCRN", 1,  9, matrix_Color.FontRed, matrix_Color.FontGreen, matrix_Color.FontBlue, matrix_Color.FontWhite, true);
    matrix_Show();                                                // Anzeige aktualisieren
    matrix_NextScreen();
    while (matrix_Keypress())
    {
      IoT_Idle();
      delay(10);
    }
    matrix_Clear();
    matrix_Show();                                                // Anzeige aktualisieren
  }
  else
  {
    bool empty = matrix_DisplayMatrix();
    unsigned long now = millis();
    IoT_WebServer.handleClient();
    if (now - LastWLANtime > WIFI_TIMEOUT)
    {
      Serial.print("WLAN-Verbindung wird geprueft... ");
      if (IoT_WLANstatus() != "Connected")
      {
        Serial.println("Verbindung muss neu aufgebaut werden...");
        matrix_WlanAutoConnect();
      }
      else
      {
        Serial.println("OK");
      }
      LastWLANtime = now;
    }
  }
  while (s == second())
  {
    IoT_WebServer.handleClient();                                 // Bediene die http Anfragen
    delay(10);
  }
}

String StandardHeader ()
{
  String ColorHexVal = "#000000";
  String title = "ESP_Matrix - IP: " + IoT_WLANaddress(false);
  String content = "<!DOCTYPE html>";
  content += "<html>";
  content += "<head>";
  content += "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">";
  content += "<title>" + title + "</title>";
  content += "<style> body { background-color: #FFFFFF; font-family: Menlo, Monaco, Courier, monospace; Color: " + ColorHexVal + "; } </style>";
  content += "</head>";
  content += "<body>";
  content += "<img src='http://newsletter.allnet.de/mailing/newsletteruploads/pm/default/logos/own/logo-allnet.png' alt='Allnet Logo'>";
  content += "<h1> " + title + " </h1>";
  content += "<h2> 16 x 16 RGBW-LED Matrix";
  content += "<h2>" + matrix_ID + "</h2>";
  content += "<h2>" + dayname(weekday() - 1) + " der " + IoT_NTPdate() + " um " + IoT_NTPtime() + "</h2>";
  return (content);
}

void handleRoot ()
{
  String ColorHexVal     = "#000000";
  String ipaddr_web      = IoT_WLANaddress(false);
  String ipaddr_gateway  = IoT_WLANgateway(false);
  String ipaddr_subnet   = IoT_WLANsubnet(false);
  String staticInfo = " (dynamisch)";
  if (matrix_StaticIP == 1)
  {
    staticInfo = " (statisch)";
  }
  int sec = millis() / 1000;
  int min = sec / 60;
  int hr = min / 60;
  String content = StandardHeader();
  content += cleanhtml(fillcenter(" Aktuelle Netzwerk-Einstellungen ", "—", 85)) + "</p> <p> </p>";
  content +=
   "<p>" + cleanhtml("IP-Addresse   : " + ipaddr_web)      + "</p>\
    <p>" + cleanhtml("Gateway       : " + ipaddr_gateway)  + "</p>\
    <p>" + cleanhtml("Subnetzmaske  : " + ipaddr_subnet)   + "</p> <p> </p>";
  content +=
   "<p>" + cleanhtml(fillcenter(" Netzwerk ", "—", 85)) + "</p> <p> </p>" +
    IoT_WebFormOpen("NetworkChange") +
   "<p>" + cleanhtml("       IP-Adresse     Gateway    Subnetzmaske") + "</p>" +
   IoT_WebFormSubmitButton("Ändern") +
   IoT_WebInput(" ", "ip01", ipaddr_web, 15) +
   IoT_WebInput(" ", "ip02", ipaddr_gateway, 15) +
   IoT_WebInput(" ", "ip03", ipaddr_subnet, 15) + staticInfo +
   IoT_WebFormClose() +
   IoT_WebFormActionButton ("DynamicIP", "Dynamische IP-Adresse") + "<p> </p>";
  content +=
   "<p>—" + cleanhtml(fillcenter(" Gerätename ", "—", 85)) + "</p> <p> </p>" + // Ein "—" mehr wegen dem Umlaut im Titel
    IoT_WebFormOpen("NameChange") +
    IoT_WebFormSubmitButton("Ändern") +
    IoT_WebInput(" ", "devname", matrix_ID, 64) +
    IoT_WebFormClose();
  content +=
   "<p>" + cleanhtml(fillcenter(" Einstellungen ", "—", 85)) + "</p> <p> </p>\
    " + IoT_WebFormActionButton ("StandardPrefs", "Standard Einstellungen wiederherstellen") + "\
    " + IoT_WebFormActionButton ("SavePrefs", "Aktuelle Einstellungen dauerhaft speichern") + "<p> </p>\
    " + IoT_WebFormActionButton ("MatrixKonfig", "Matrix-Anzeige konfigurieren") + "<p> </p>\
    " + IoT_WebFormActionButton ("ALL3500", "ALL-3500 Sensoren konfigurieren") + "<p> </p>\
    " + IoT_WebFormActionButton ("ColorKonfig", "Farbeinstellungen konfigurieren") + "<p> </p>";
  content +=  IoT_WebFormActionButton ("Reload", "Seite neu laden");
  content += "</body>";
  content += "</html>";
  IoT_WebServer.send(200, "text/html", content);   // HTTP-Statuscode 200 = OK (Anfrage wurde erfolgreich bearbeitet)
}

void handleMatrixKonfig ()
{
  String ColorHexVal     = "#000000";
  String ipaddr_web      = IoT_WLANaddress(false);
  String ipaddr_gateway  = IoT_WLANgateway(false);
  String ipaddr_subnet   = IoT_WLANsubnet(false);
  String staticInfo = " (dynamisch)";
  if (matrix_StaticIP == 1)
  {
    staticInfo = " (statisch)";
  }
  int sec = millis() / 1000;
  int min = sec / 60;
  int hr = min / 60;
  String content = StandardHeader();
  content += cleanhtml(fillcenter(" Modul-Einstellungen ", "—", 85)) + "</p> <p> </p>";
  content += IoT_WebFormOpen("MatrixChange");
  for (int i = 0; i < 16; i++)
  {
    String num = cleanhtml(strform(i, 32, 2, false));
    String ind1 = strform(i * 2, 48, 2, false);
    String ind2 = strform(i * 2 + 1, 48, 2, false);
    content += "M " + num + " oben";
    content += IoT_WebInput(": ", "mx" + ind1, str(matrix_VirtualScreen[i * 2]), 4);
    content += IoT_WebInput(" (", "tx" + ind1, matrix_VirtualHeader[i * 2], 6);
    content += ") unten";
    content += IoT_WebInput(": ", "mx" + ind2, str(matrix_VirtualScreen[i * 2 + 1]), 4);
    content += IoT_WebInput(" (", "tx" + ind2, matrix_VirtualHeader[i * 2 + 1], 6);
    content += ")";
    content += "<br> ";
  }
  content += "<br> ";
  content += IoT_WebFormSubmitButton("Ändern");
  content += IoT_WebFormClose();
  if (matrix_ShowModules)
  {
    content += "<p>—" + cleanhtml(fillcenter(" Verfügbare Module ", "—", 85)) + "</p> <p> </p>"; // Ein "—" mehr wegen dem Umlaut im Titel
    content += cleanhtml(" 0 = Kein Modul<br>");
    content += cleanhtml(" 1 = Text-Anzeige (4 Zeichen)<br>");
    content += cleanhtml(" 2 = Zeit (hh:mm)<br>");
    content += cleanhtml(" 3 = Zeit (ss)<br>");
    content += cleanhtml(" 4 = Datum (dd.mm)<br>");
    content += cleanhtml(" 5 = Datum (jjjj)<br>");
    content += cleanhtml(" 6 = Temperatur<br>");
    content += cleanhtml(" 7 = Luftfeuchtigkeit<br>");
    content += cleanhtml(" 8 = BitCoin (Währungskürzel)<br>");
    if (all3500_SensorCount > 0)
    {
      for (int i = 0; i < all3500_SensorCount; i++)
      {
        content += str(16 + i);
        content += " = ";
        content += cleanhtml(left(cleanasc(all3500_SensorName[i]), 25));
        content += " (";
        content += cleanhtml(left(cleanasc(all3500_SensorInfo[i]), 25));
        content += ")";
        content += "<br> ";
      }
    }
    content += "<br> ";
    content +=  IoT_WebFormActionButton ("ModulesOff", "Verfügbare Module ausblenden");
  }
  else
  {
    content += "<br> ";
    content +=  IoT_WebFormActionButton ("ModulesOn", "Verfügbare Module einblenden");  
  }
  content += "<p>" + cleanhtml(fillcenter(" Startseite ", "—", 85)) + "</p> <p> </p>";
  content +=  IoT_WebFormActionButton ("Reload", "Startseite");
  content += "</body>";
  content += "</html>";
  IoT_WebServer.send(200, "text/html", content);   // HTTP-Statuscode 200 = OK (Anfrage wurde erfolgreich bearbeitet)
}

void handleMatrixChange ()                                                // Matrix-Konfiguration ändern
{
  for (int i = 0; i < 16; i++)
  {
    String num = cleanhtml(strform(i, 32, 2, false));
    String ind1 = strform(i * 2, 48, 2, false);
    String ind2 = strform(i * 2 + 1, 48, 2, false);
    matrix_VirtualScreen[i * 2] = val(IoT_WebGetField("mx" + ind1));
    matrix_VirtualHeader[i * 2] = IoT_WebGetField("tx" + ind1);
    matrix_VirtualScreen[i * 2 + 1] = val(IoT_WebGetField("mx" + ind2));
    matrix_VirtualHeader[i * 2 + 1] = IoT_WebGetField("tx" + ind2);
  }
  for (int i = 0; i < 32; i++)
  {
    int m = matrix_VirtualScreen[i];
    if ((m == 1) || (m == 8))                                             // Text-Modul oder BitCoin Modul
    {
      matrix_VirtualHeader[i] = left(ucase(matrix_VirtualHeader[i]), 4);
    }
    else
    {
      matrix_VirtualHeader[i] = "";
    }
    if ((m < 1) || (m > 63))                                              // Illegale Modulnummer
    {
      matrix_VirtualScreen[i] = 0;
    }
    if ((m > 8) && (m < 16))                                              // Illegale Modulnummer
    {
      matrix_VirtualScreen[i] = 0;
    }
  }
  handleMatrixKonfig();                                                   // Startseite anzeigen
}

void handleModulesOn ()
{
  matrix_ShowModules = true;
  handleMatrixKonfig();                                                   // Startseite anzeigen
}

void handleModulesOff ()
{
  matrix_ShowModules = false;
  handleMatrixKonfig();                                                   // Startseite anzeigen
}

void handleALL3500 ()
{
  String content = StandardHeader();
  content += cleanhtml(fillcenter(" ALL-3500 URL ", "—", 85)) + "</p> <p> </p>";
  content += IoT_WebFormOpen("ALL3500Change");
  content += IoT_WebFormSubmitButton("Ändern");
  content += IoT_WebInput(" ", "url", matrix_ALL3500URL, 75);
  if (all3500_SensorCount == 0)
  {
    if (matrix_ALL3500URL != "")
    {
      content += " (ungültige URL)";
    }
  }
  else
  {
    content += " (" + str(all3500_SensorCount) + " Sensoren)";
  }
  content += IoT_WebFormClose();
  content += "<p>—" + cleanhtml(fillcenter(" Verfügbare Module ", "—", 85)) + "</p> <p> </p>"; // Ein "—" mehr wegen dem Umlaut im Titel
  content += cleanhtml(" 0 = Kein Modul<br>");
  content += cleanhtml(" 1 = Text-Anzeige (4 Zeichen)<br>");
  content += cleanhtml(" 2 = Zeit (hh:mm)<br>");
  content += cleanhtml(" 3 = Zeit (ss)<br>");
  content += cleanhtml(" 4 = Datum (dd.mm)<br>");
  content += cleanhtml(" 5 = Datum (jjjj)<br>");
  content += cleanhtml(" 6 = Temperatur<br>");
  content += cleanhtml(" 7 = Luftfeuchtigkeit<br>");
  content += cleanhtml(" 8 = BitCoin (Währungskürzel)<br>");
  if (all3500_SensorCount > 0)
  {
    for (int i = 0; i < all3500_SensorCount; i++)
    {
      content += str(16 + i);
      content += " = ";
      content += cleanhtml(left(cleanasc(all3500_SensorName[i]), 25));
      content += " (";
      content += cleanhtml(left(cleanasc(all3500_SensorInfo[i]), 25));
      content += ")";
      content += "<br> ";
    }
  }
  content += "<p>" + cleanhtml(fillcenter(" Startseite ", "—", 85)) + "</p> <p> </p>";
  content +=  IoT_WebFormActionButton ("Reload", "Startseite");
  content += "</body>";
  content += "</html>";
  IoT_WebServer.send(200, "text/html", content);   // HTTP-Statuscode 200 = OK (Anfrage wurde erfolgreich bearbeitet)
}

void handleALL3500Change ()                                         // ALL-3500 URL ändern
{
  matrix_ALL3500URL = IoT_WebGetField("url");
  all3500_ReadSensorData(matrix_ALL3500URL, true);
  handleALL3500();                                                  // Startseite anzeigen
}

void handleColorKonfig ()                                           // Farbkonfiguration
{
  String content = StandardHeader();
  content += cleanhtml(fillcenter(" Farbeinstellungen ", "—", 85)) + "</p> <p> </p>";
  content += IoT_WebFormOpen("ColorChange");
  content += cleanhtml("RGBW Schrift     ");
  content += IoT_WebInput(" R:", "fr", str(matrix_Color.FontRed), 5);
  content += IoT_WebInput(" G:", "fg", str(matrix_Color.FontGreen), 5);
  content += IoT_WebInput(" B:", "fb", str(matrix_Color.FontBlue), 5);
  content += IoT_WebInput(" W:", "fw", str(matrix_Color.FontWhite), 5);
  content += "<br>";
  content += cleanhtml("RGBW Trennzeichen");
  content += IoT_WebInput(" R:", "tr", str(matrix_Color.SeparatorRed), 5);
  content += IoT_WebInput(" G:", "tg", str(matrix_Color.SeparatorGreen), 5);
  content += IoT_WebInput(" B:", "tb", str(matrix_Color.SeparatorBlue), 5);
  content += IoT_WebInput(" W:", "tw", str(matrix_Color.SeparatorWhite), 5);
  content += "<br> <br>";
  content += cleanhtml("Helligkeit       ");
  content += IoT_WebInput(" L:", "br", str(matrix_Color.Brightness), 5);
  content += "<br> <br>";
  content += IoT_WebFormSubmitButton("Ändern");
  content += "<br>";
  content += IoT_WebFormClose();
  content += "<p>" + cleanhtml(fillcenter(" Startseite ", "—", 85)) + "</p> <p> </p>";
  content +=  IoT_WebFormActionButton ("Reload", "Startseite");
  content += "</body>";
  content += "</html>";
  IoT_WebServer.send(200, "text/html", content);   // HTTP-Statuscode 200 = OK (Anfrage wurde erfolgreich bearbeitet)
}

void handleColorChange ()                                           // Farbkonfiguration ändern
{
  matrix_Color.FontRed        = val(IoT_WebGetField("fr"));
  matrix_Color.FontGreen      = val(IoT_WebGetField("fg"));
  matrix_Color.FontBlue       = val(IoT_WebGetField("fb"));
  matrix_Color.FontWhite      = val(IoT_WebGetField("fw"));
  matrix_Color.SeparatorRed   = val(IoT_WebGetField("tr"));
  matrix_Color.SeparatorGreen = val(IoT_WebGetField("tg"));
  matrix_Color.SeparatorBlue  = val(IoT_WebGetField("tb"));
  matrix_Color.SeparatorWhite = val(IoT_WebGetField("tw"));
  matrix_Color.Brightness     = val(IoT_WebGetField("br"));
  matrix_Pixel.setBrightness(matrix_Color.Brightness);              // Helligkeit (0..255)
  handleColorKonfig();                                              // Startseite anzeigen
}

void handleDynamicIP ()                                             // Dynamische IP-Adresse
{
  matrix_StaticIP = 0;
  IPAddress ip1 = IPAddress(IoT_dynamicIP_address0, IoT_dynamicIP_address1, IoT_dynamicIP_address2, IoT_dynamicIP_address3);
  IPAddress ip2 = IPAddress(IoT_dynamicIP_gateway0, IoT_dynamicIP_gateway1, IoT_dynamicIP_gateway2, IoT_dynamicIP_gateway3);
  IPAddress ip3 = IPAddress(IoT_dynamicIP_subnet0,  IoT_dynamicIP_subnet1,  IoT_dynamicIP_subnet2,  IoT_dynamicIP_subnet3);
  WiFi.config(ip1, ip2, ip3);                                       // Parameter: IP, Gateway, Subnet, DNS
  handleRoot();                                                     // Startseite anzeigen
}

void handleNetworkChange ()                                         // Statische IP-Adresse
{
  matrix_StaticIP = 1;
  IPAddress ip1 = IPval(IoT_WebGetField("ip01"));                   // Statische IP-Adresse
  IPAddress ip2 = IPval(IoT_WebGetField("ip02"));                   // Gateway/Router IP-Adresse
  IPAddress ip3 = IPval(IoT_WebGetField("ip03"));                   // Subnet IP-Adresse
  WiFi.config(ip1, ip2, ip3);                                       // Parameter: IP, Gateway, Subnet, DNS
  handleRoot();                                                     // Startseite anzeigen
}

void handleNameChange ()                                            // Name ändern
{
  matrix_ID = IoT_WebGetField("devname");                           // Geräte-Name
  handleRoot();                                                     // Startseite anzeigen
}

void handleStandardPrefs ()
{
  matrix_ParameterNew();                                            // Aktuelle Konfiguration in den Parameter-Block schreiben
  matrix_ParameterApply();                                          // Parameter-Block in die aktuelle Konfiguration übernehmen
  handleRoot();                                                     // Startseite anzeigen
}

void handleSavePrefs ()
{
  matrix_ParameterCreate();                                         // Aktuelle Konfiguration in den Parameter-Block schreiben
  matrix_ParameterSave();                                           // Parameter-Block in das EEPROM sichern
  IoT_EEPROMupdate();                                               // EEPROM aktualisieren
  handleRoot();                                                     // Startseite anzeigen
}
